@@ -9,6 +9,7 @@ import warnings
 import time
 import numpy as np
 import torch
+from trident.data.dataset import MaskDataset, ZipDataset
 
 from trident import context
 import numbers
@@ -16,7 +17,6 @@ from trident.backend.common import *
 from trident.backend.load_backend import *
 from trident.backend.pillow_backend import image2array
 from trident.callbacks.callback_base import CallbackBase
-from trident.data.dataset import MaskDataset, ImageDataset, ZipDataset
 from trident.data.image_common import image_backend_adaption
 from trident.data.mask_common import label2color
 from trident.misc.ipython_utils import is_in_ipython, is_in_colab
