@@ -23,6 +23,7 @@ from typing import List, TypeVar, Tuple, Union, Optional, Generic, Iterable, Ite
 import cv2
 import numpy as np
 from skimage import color
+from trident.backend.numpy_ops import DTYPE_MAPPING
 
 from trident.backend.opencv_backend import file2array
 from trident.data.vision_transforms import Unnormalize
@@ -1234,18 +1235,31 @@ class Iterator(object):
 
     def update_signature(self):
         datasets = self.get_datasets()
-        results=self[0]
+        results = self[0]
+        self.data_template.clear()
         self._signature = Signature(name='data_provider')
 
-        for ds, result,value in zip(datasets, results.key_list,results.value_list):
-            spec = TensorSpec.tensor_to_spec(value, object_type=ds.object_type,need_exclude_batch_axis=True,is_singleton=True, name=ds.symbol)
-            self._signature.outputs[ds.symbol] = spec
+        for ds, result, value in zip(datasets, results.key_list, results.value_list):
+            spec = copy.deepcopy(ds._element_spec)
+            if spec.name == ds.symbol:
+                if isinstance(value,np.ndarray):
+                    dtype=DTYPE_MAPPING[value.dtype.type]
+                elif isinstance(value,numbers.Integral):
+                    dtype=numbers.Integral
+                elif isinstance(value,numbers.Real):
+                    dtype=numbers.Real
+                spec.shape = tensor_to_shape(to_tensor(value), need_exclude_batch_axis=True, is_singleton=True)
+                spec.object_type = ds.object_type
+                spec.dtype = dtype
+
+                ds._element_spec = spec
+                self.data_template[spec] = None
+                self._signature.outputs[ds.symbol] = spec
         self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
         return self._signature
 
 
-
     def print_statistics(self):
         print('avg. process time: {0:.5f}'.format(self.pass_time_spend / float(builtins.max(1, self.pass_cnt))))
         print('avg. batch time: {0:.5f}'.format(self._batch_size * self.pass_time_spend / float(builtins.max(1, self.pass_cnt))))
