@@ -19,7 +19,7 @@ import torch.nn as nn
 import torch.nn.functional as F
 
 from matplotlib.collections import PolyCollection
-from torch._six import container_abcs
+from collections import abc
 from torch.nn import init
 from torch.nn.parameter import Parameter
 from trident.backend.opencv_backend import image2array
